local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local LSM = LibStub("LibSharedMedia-3.0")

local Boss = RW.bossPrototype
local messageProt = {}
local mt = {__index = messageProt}

local Party = RW:GetPlugin("Party")
local BITS = RW.BITS

-- If text is a number this warning is considered not a custom one
-- If text is not a number then it is custom and it can be possibly sent as a table,
-- then the first value in the table will be the actual text, and the second oen will be the option text as its show in the GUI
-- 3rd value in the table will be the tooltip text
local spell
function Boss:_Message(mType, text, icon, optionName, default, color)
	if type(text) == "number" then 
		spell = text
		return self:_Message(mType, GetSpellInfo(text), icon or text, optionName or text, default, color) 
	end
	if type(icon) == "number" then
		spell = icon
		return self:_Message(mType, text, select(3, GetSpellInfo(icon)), optionName or icon, default, color)
	end
	
	optionName = type(optionName) == "number" and RW.SpellName[optionName] or optionName
	
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	self:AddToCategory(nil, optionName)
	
	local defaults = self:AddDefaults(optionName, "MESSAGE", mType ~= "CUSTOM" and mType or text[2], default)
	defaults.color = color
	defaults.sound = "RW: Click"
	
	local base = self:GetBaseAlertObject(optionName)
	if not base then
		base = {
			icon = icon,			
			id = optionName,
			spell = spell,
			mod = self,
			scheduled = {},
		}
		tinsert(self.BaseAlertObjects, base)
	end
	
	local obj = setmetatable({
		mainType = "MESSAGE",
		type = mType,	
		text = type(text) == "table" and text[1] or text,
		optText = (mType == "CUSTOM" and type(text) == "table") and text[2] or nil,
		base = base,
	}, mt)
	
	self.AlertObjects.MESSAGE = self.AlertObjects.MESSAGE or {}
	tinsert(self.AlertObjects.MESSAGE, obj)
	
	spell = nil
	
	return obj
end

function Boss:Message(...)
	return self:_Message("NORMAL", ...)
end

function Boss:CastMessage(time, ...)
	return self:_Message("CAST", time, ...)
end

function Boss:TargetMessage(...)
	return self:_Message("TARGET", ...)
end

function Boss:TargetStackMessage(...)
	return self:_Message("TARGET_STACK", ...)
end

function Boss:PhaseMessage(text, icon, option, ...)
	return self:_Message("PHASE", nil, icon or [[Interface\Icons\Achievement_PVP_A_01]], option or "Phase Switch", ...)
end

function Boss:CustomMessage(text, optText, ...)
	--@debug
	assert(type(text) == "string", self:GetName()..": CustomTimer, variable \"text\" is not of type text")
	assert(type(optText) == "string", self:GetName()..": CustomTimer, variable \"optText\" is not of type text")
	--@end-debug
	return self:_Message("CUSTOM", {text, optText}, ...)
end

function messageProt:SetIcon(icon)
	self.icon = icon
end

function messageProt:Show(...)	
	if not self.base.mod.DB[self.base.id].enabled then return end
	if not self.base.mod.DB[self.base.id].MESSAGE[self.optText or self.type].enabled then return end
	
	local text
	if self.optText then	
		text = RW:Format(self.text, ...)
	else
		text = RW:Format(L["FORMAT_MESSAGE_"..self.type], self.text or ..., ...)
	end

	if self.type == "PHASE" and self.base.icon:find("Interface\\Icons\\Achievement_PVP_A_0") then
		local phase = ...
		self.icon = "Interface\\Icons\\Achievement_PVP_A_0"..phase..".blp"
	end
	
	text = RW:ColorNames(text)
	
	local tc = self.base.mod.DB[self.base.id].MESSAGE[self.optText or self.type].color
	local c 
	if tc then c = tc else c = {1, 1, 1, 1} end
	self.base.mod:SendMessage("RW_Message", text, c[1], c[2], c[3], c[4], self.base.icon)
	self.base.mod:Msg((self.icon and "|T"..self.base.icon..":20|t " or "")..text)
	PlaySoundFile(LSM:Fetch("sound", self.base.mod.DB[self.base.id].MESSAGE[self.optText or self.type].sound))
end
messageProt.Start = messageProt.Show

local function showMessage(message, ...)
	message:Show(...)
end

function messageProt:Schedule(stime, info, ...)
	self.base.mod:Schedule("MESSAGE"..self.base.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.base.scheduled, "MESSAGE"..self.base.id..(info or ""))
end

function messageProt:ScheduleRepeating(stime, now, info, ...)
	self.base.mod:ScheduleRepeating("MESSAGE"..self.base.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.base.scheduled, "MESSAGE"..self.base.id..(info or ""))
end

function messageProt:Unschedule(extraID)
	if not extraID then
		for i, messageID in pairs(self.base.scheduled) do
			self.base.mod:Unschedule(messageID)
		end
		wipe(self.base.scheduled)
	else
		self.mod:Unschedule("MESSAGE"..self.base.id..(extraID or ""))
		RW:TRemove(self.scheduled, "MESSAGE"..self.base.id..(extraID or ""))
	end
end
messageProt.Cancel = messageProt.Unschedule

-- Options
function messageProt:GetOption()
	local Options = RW:GetPlugin("Options")
	local boss = self.base.mod
	local AceGUI = LibStub("AceGUI-3.0")

	local subGroup = AceGUI:Create("OptionFrame")
	subGroup:SetFullWidth(true)
	subGroup:SetLayout("flow")
	
	local check = AceGUI:Create("CheckBox")
	check:SetRelativeWidth(0.26)
	if self.type == "CUSTOM" then
		subGroup:SetTitle(self.optText)				
	else
		subGroup:SetTitle(L["MESSAGE_"..self.type])
		check:SetCallback("OnEnter", Options.OptionHelpers.OnOptionEnter)
		check:SetCallback("OnLeave",  Options.OptionHelpers.OnOptionLeave)
	end			
	check:SetUserData("obj", self)
	check:SetCallback("OnValueChanged", Options.OptionHelpers.OnValueChanged)
	check:SetValue(boss.DB[self.base.id].MESSAGE[self.optText or self.type].enabled)
	check:SetLabel("Enable")
	
	local color = AceGUI:Create("ColorPicker")
	color:SetRelativeWidth(0.30)
	local colorvalue = boss.DB[self.base.id].MESSAGE[self.optText or self.type].color or nil
	if colorvalue then 
		color:SetColor(unpack(colorvalue))
	end
	color:SetLabel("Color")
	color:SetUserData("obj", self)
	color:SetCallback("OnValueConfirmed", Options.OptionHelpers.OnColorSelected)
	
	local drop = AceGUI:Create("LSM30_Sound")
	drop:SetRelativeWidth(0.39)
	drop:SetList()
	drop:SetLabel("Sound")
	drop:SetUserData("obj", self)
	drop:SetUserData("key", "sound")
	drop:SetCallback("OnValueChanged", Options.OptionHelpers.OnKeyValueChanged)
	drop:SetValue(boss.DB[self.base.id].MESSAGE[self.optText or self.type].sound)
	
	subGroup:AddChild(check)
	subGroup:AddChild(color)
	subGroup:AddChild(drop)
	
	return subGroup
end 