local mod = RW:Boss("Lady Deathwhisper", "Icecrown Citadel", "Icecrown Citadel", "The Lower Spire")
local L

-- 3.3?
if not QueryQuestsCompleted then return end

---------------------
-- Variables

-- Lady
local tDominateDur, tDominateCD, mDominate
local tDNDCD, tDNDDur, wDND

-- Adds
local tNextAdds
local iEmpowerment, wEmpowerment
local wTorpor, mTorpor, tTorporDur
local wShroud
local wMartyr, iMartyr, tMartyrCast
local wDeformed
local iTransform, wTransform

-- General
local tBerserk
local mPhase, wPhase

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L.name = "Lady Deathwhisper"
		
		L.YellPull = "What is this disturbance? You dare trespass upon this hallowed ground? This shall be your final resting place!"
		L.YellWin = "All part of the Master's plan.... Your end is inevitable...."
		
		L.cat_CultAdherent = "Cult Adherent"
		L.cat_CultFanatic = "Cult Fanatic"
		
		L.mana_barrier = GetSpellInfo(70842)
		L.phasewarning = "Phase 2 Starting!"
		L.phasewarning_opt = "Phase Switch"
		
		L.next_adds = "New Adds"
		L.next_adds_opt = "Timer for when the next set of adds spawn"
		L.next_adds_group = "Add spawns"
		
		L.deformed_name = "Deformed Fanatic"
		L.deformed_name_opt = "Warn for new Deformed Fanatics"
	end
	L = mod:GetLocale()
	mod.L = L
	
	mod:SetTrigger(L.YellPull, 36855)
	mod:SetWin(L.YellWin)
	
	self:AddToCategory(L.name, 71289, 72109)
	self:AddToCategory(L.cat_CultAdherent, 70901, 71237, 70768, 72499)
	
	self:AddBoolOption("ManaShieldHelath", "Show Mana Barrier health", L.name, true, select(3, GetSpellInfo(70842)))
	
	tNextAdds		= self:CustomTimer(60, L.next_adds, L.next_adds_opt, [[Interface\Icons\Achievement_PVP_H_10]], L.next_adds_group)
	tDominateDur	= self:DurTargetTimer(20, 71289)
	tDominateCD		= self:CDTimer(40, 71289)
	tDNDCD			= self:CDTimer(20, 72109)
	tDNDDur			= self:DurTimer(10, 72109)
	tBerserk		= self:BerserkTimer(600)
	tMartyrCast		= self:CastTimer(72499)
	
	wDND			= self:RunawayWarning(72109)
	wPhase			= self:CustomWarning(L.phasewarning, L.phasewarning, [[Interface\Icons\Achievement_PVP_A_02.blp]], L.phasewarning_opt)
	wTransform		= self:CastWarning(70900)
	wEmpowerment	= self:CastWarning(70901)
	wTorpor			= self:Warning(71237)
	wShroud			= self:CastWarning(70768)
	wMartyr			= self:CastWarning(72499)
	
	mDominate		= self:TargetMessage(71289)
	mPhase 			= self:PhaseMessage()
	
	iEmpowerment	= self:Icon("CROSS", 70901, false, nil, "SQUARE")
	iMartyr			= self:Icon("CIRCLE", 72499, false)
	iTransform		= self:Icon("SKULL", 70900, false)
end

----------------------
-- Locals

local manaLeft
local mcTargets = {}
local isCaster = mod.pClass == "MAGE" or mod.pClass == "WARLOCK" or mod.pClass == "DRUID" or mod.pClass == "PRIEST" or mod.pClass == "DEATHKNIGHT"


-- Health frames
local function ManaBarrierUpdate()
	local uID = mod:UIDFromCID(36855)
	if uID then
		manaLeft = UnitMana(uID) / UnitManaMax(uID)
	end
	return manaLeft
end

local function addHandler()
	tNextAdds:Start()
	mod:Schedule("Adds", 60, addHandler)
end

----------------------
-- Start

function mod:OnEngage(trigger)
	-- Lady
	self:Track("Dominate", 71289)
	self:Track("DND", 72109, 72108, 71001)
	self:Track("ManaBarrier", 70842)
	-- Adds
	self:Track("DarkTransform", 70900)
	self:Track("Empowerment", 70901)
	self:Track("Torpor", 71237)
	self:Track("Shroud", 70768)
	self:Track("Martyr", 72499)
	
	tBerserk:Start()
	
	if self.db.profile.ManaShieldHelath then
		self:SendMessage("RW_AddHealthFrame", ManaBarrierUpdate, L.mana_barrier)
	end
	
	tNextAdds:Start(7)
	mod:Schedule("Adds", 7, addHandler)
	
	manaLeft = 1
	wipe(mcTargets)
end

local function showMCTargets()
	mDominate:Show(table.concat(mcTargets, "<, >"))
	wipe(mcTargets)
end

function mod:Dominate(event, args)
	if event == "SPELL_AURA_APPLIED" then
		tDominateDur:Start(args.dName)
		tDominateCD:Start()
		mcTargets[#mcTargets + 1] = args.dName
		self:Schedule("MCTargets", 0.2, showMCTargets)
		
	elseif event == "SPELL_AURA_REMOVED" then
		tDominateDur:Cancel(args.dName)
	end
end

function mod:DND(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tDNDCD:Start()
		tDNDDur:Start()
		
	elseif event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wDND:Show()
		end
	end
end

function mod:ManaBarrier(event, args)
	if event == "SPELL_AURA_REMOVED" then
		self:Unschedule("Adds")
		tNextAdds:Stop()
		mPhase:Show(2)
		wPhase:Show()
		self:SendMessage("RW_RemoveAllHealthFrames")
	end
end

-- Dark Transformation
do
	local function transformScanner(gUID, scanNR)
		local target = mod:UIDFromGUID(gUID)
		
		if target then
			iTransform:Show(target)
		elseif scanNR < 10 then
			mod:Schedule("TransformScan", 0.5, transformScanner, gUID or "", scanNR + 1)
		end
	end

	function mod:DarkTransform(event, args)
		if event == "SPELL_CAST_START" then
			wTransform:Show()
			self:Schedule("TransformScan", 0.2, transformScanner, args.sGUID or "", 1)
		end
	end
end

-- Dark Empoverment
do
	local function empScanner(gUID, scanNR)
		local target = mod:UIDFromGUID(gUID)

		if target then
			iEmpowerment:Show(gUID)
		end
		
		if not target and scanNR < 10 then
			mod:Schedule("EmpScanner", 0.2, empScanner, gUID or "", scanNR + 1)
		end
	end

	function mod:Empowerment(event, args)
		if event == "SPELL_CAST_START" then
			wEmpowerment:Show()
			self:Schedule("EmpScanner", 0.2, empScanner, args.sGUID or "", 1)
		end
	end
end

function mod:Torpor(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wTorpor:Show()
		end
	end
end

function mod:Shroud(event, args)
	if event == "SPELL_CAST_START" then
		if UnitGUID("target") == args.sGUID and self.isCaster then
			wShroud:Show()
		end
	end
end

do
	local function martyrScanner(gUID, scanNR)
		local target = mod:UIDFromGUID(gUID)

		iMartyr:Show(gUID)
		
		if not target and scanNR < 5 then
			mod:Schedule("MartyrScanner", 1, martyrScanner, gUID or "", scanNR + 1)
		end
	end
	
	function mod:Martyr(event, args)
		if event == "SPELL_CAST_START" then
			tMartyrCast:Start()
			wMartyr:Show()
			self:Schedule("MartyrScanner", 0.2, martyrScanner, args.sGUID or "", 1)
		end
	end
end